<?php

/**
 * @file
 * Admin pages of the CDEK API module.
 */

/**
 * Indicates no caching.
 */
const CDEK_API_CACHE_TYPE_NONE = 1;

/**
 * Indicates permanent caching.
 */
const CDEK_API_CACHE_TYPE_PERMANENT = 2;

/**
 * Indicates caching with a custom cache lifetime.
 */
const CDEK_API_CACHE_TYPE_CUSTOM = 3;

/**
 * Provides a form to manage settings.
 *
 * @param array $form
 *   The form structure.
 *
 * @return array
 *   The form structure.
 *
 * @see cdek_api_settings_form_submit()
 */
function cdek_api_settings_form(array $form) {
  $config = CdekApi::getConfig();

  $form['basic'] = [
    '#type' => 'fieldset',
    '#title' => t('Basic'),
  ];

  $form['advanced'] = [
    '#type' => 'fieldset',
    '#title' => t('Advanced'),
  ];

  // Account.
  $form['basic']['account'] = [
    '#type' => 'textfield',
    '#title' => t('Account'),
    '#description' => t('Contractor identifier.'),
    '#default_value' => $config['account'],
    '#size' => 40,
  ];

  // Secure password.
  $form['basic']['password'] = [
    '#type' => 'textfield',
    '#title' => t('Secure password'),
    '#description' => t('Security code provided by CDEK.'),
    '#default_value' => $config['password'],
    '#size' => 40,
  ];

  // Test mode.
  $form['basic']['test_mode'] = [
    '#type' => 'checkbox',
    '#title' => t('Test mode'),
    '#default_value' => $config['test_mode'],
  ];

  // Timeout.
  $form['advanced']['request_timeout'] = [
    '#type' => 'textfield',
    '#title' => t('Timeout'),
    '#description' => t('Float representing the maximum number of seconds the API request may take. Use 0 to wait indefinitely.'),
    '#default_value' => $config['request_timeout'],
    '#element_validate' => ['cdek_api_element_validate_float_positive'],
    '#size' => 5,
    '#required' => TRUE,
  ];

  $cache_lifetime = $config['cache_lifetime'];
  if ($cache_lifetime === NULL) {
    $cache_type = CDEK_API_CACHE_TYPE_NONE;
  }
  elseif ($cache_lifetime === CACHE_PERMANENT) {
    $cache_type = CDEK_API_CACHE_TYPE_PERMANENT;
    $cache_lifetime = NULL;
  }
  else {
    $cache_type = CDEK_API_CACHE_TYPE_CUSTOM;
  }

  // Cache type.
  $form['advanced']['cache_type'] = [
    '#type' => 'radios',
    '#title' => t('Cache type'),
    '#options' => [
      CDEK_API_CACHE_TYPE_NONE => t('None'),
      CDEK_API_CACHE_TYPE_PERMANENT => t('Permanent'),
      CDEK_API_CACHE_TYPE_CUSTOM => t('Custom'),
    ],
    '#default_value' => $cache_type,
    '#required' => TRUE,
  ];

  $conditions = [
    ':input[name="cache_type"]' => [
      'value' => CDEK_API_CACHE_TYPE_CUSTOM,
    ],
  ];

  // Cache lifetime.
  $form['advanced']['cache_lifetime'] = [
    '#type' => 'textfield',
    '#title' => t('Cache lifetime'),
    '#description' => t('Cache lifetime in minutes.'),
    '#default_value' => $cache_lifetime,
    '#states' => [
      'visible' => $conditions,
      'required' => $conditions,
      'enabled' => $conditions,
    ],
    '#size' => 20,
    '#element_validate' => [
      'cdek_api_element_validate_cache_lifetime',
      'element_validate_integer_positive',
    ],
  ];

  // Button to clear the cache.
  $form['advanced']['clear_cache'] = [
    '#type' => 'submit',
    '#value' => t('Clear cache'),
    '#submit' => ['cdek_api_settings_form_clear_cache_submit'],
    '#limit_validation_errors' => [],
    '#access' => $cache_type !== CDEK_API_CACHE_TYPE_NONE,
  ];

  // Custom submit callback.
  $form['#submit'][] = 'cdek_api_settings_form_submit';
  return system_settings_form($form);
}

/**
 * Form submission handler for cdek_api_settings_form().
 *
 * @param array $form
 *   The form structure.
 * @param array $form_state
 *   The current state of the form.
 *
 * @see cdek_api_settings_form()
 * @see system_settings_form_submit()
 */
function cdek_api_settings_form_submit(array $form, array &$form_state) {
  $config = CdekApi::getConfig();
  $unchanged_config = $config;

  $cache_type = (int) $form_state['values']['cache_type'];
  if ($cache_type === CDEK_API_CACHE_TYPE_PERMANENT) {
    $cache_lifetime = CACHE_PERMANENT;
  }
  elseif ($cache_type === CDEK_API_CACHE_TYPE_CUSTOM) {
    $cache_lifetime = (int) $form_state['values']['cache_lifetime'];
  }
  else {
    // Clear the cache if the API data is not cached.
    cache_clear_all('*', 'cache_cdek_api', TRUE);
    $cache_lifetime = NULL;
  }

  // Save settings.
  $config['account'] = (string) $form_state['values']['account'];
  $config['password'] = (string) $form_state['values']['password'];
  $config['test_mode'] = (bool) $form_state['values']['test_mode'];
  $config['request_timeout'] = (float) $form_state['values']['request_timeout'];
  $config['cache_lifetime'] = $cache_lifetime;

  // @see system_settings_form_submit()
  $form_state['values'] = ['cdek_api_settings' => $config];

  // Clear the cache after switching test mode.
  if ($unchanged_config['test_mode'] != $config['test_mode']) {
    cache_clear_all('*', 'cache_cdek_api', TRUE);
  }
}

/**
 * Form submission handler to clear the cache.
 */
function cdek_api_settings_form_clear_cache_submit() {
  cache_clear_all('*', 'cache_cdek_api', TRUE);
  drupal_set_message(t('The cache has been cleared.'));
}

/**
 * Form element validation handler for float elements that must be positive.
 *
 * @param array $element
 *   The element structure.
 */
function cdek_api_element_validate_float_positive(array $element) {
  $value = $element['#value'];
  if ($value !== '' && (!is_numeric($value) || floatval($value) != $value || $value <= 0)) {
    form_error($element, t('%name must be a positive float.', ['%name' => $element['#title']]));
  }
}

/**
 * Form element validation handler for cache_lifetime element.
 *
 * @param array $element
 *   The element structure.
 * @param array $form_state
 *   The current state of the form.
 */
function cdek_api_element_validate_cache_lifetime(array $element, array &$form_state) {
  $cache_type = (int) $form_state['values']['cache_type'];
  if ($cache_type === CDEK_API_CACHE_TYPE_CUSTOM && empty($element['#value'])) {
    form_error($element, t('!name field is required.', [
      '!name' => $element['#title'],
    ]));
  }
}
