<?php

/**
 * @file
 * CDEK API elements.
 */

use CdekSDK\Requests\PvzListRequest;

/**
 * Implements hook_element_info().
 *
 * @see cdek_api_element_info()
 */
function _cdek_api_element_info() {
  $types['cdek_select'] = [
    '#input' => TRUE,
    '#tree' => TRUE,
    '#value_callback' => 'cdek_select_element_value_callback',
    '#process' => ['cdek_select_element_process'],
    '#after_build' => ['cdek_select_element_after_build'],
    '#pre_render' => ['form_pre_render_conditional_form_element'],
    '#theme_wrappers' => ['cdek_select'],
    '#cdek_request' => NULL,
  ];
  $types['cdek_widget'] = [
    '#input' => TRUE,
    '#value_callback' => 'cdek_widget_element_value_callback',
    '#process' => ['cdek_widget_element_process'],
    '#pre_render' => ['cdek_widget_element_pre_render'],
    '#theme_wrappers' => ['cdek_widget'],
    '#cdek_widget_options' => [],
    '#cdek_widget_tariff_list' => [],
    '#cdek_widget_popup_title' => NULL,
  ];
  return $types;
}

/**
 * Value callback for a cdek_select form element.
 *
 * @param array $element
 *   An associative array containing the properties of the element.
 * @param mixed $input
 *   The incoming input to populate the form element. If this is FALSE,
 *   the element's default value should be returned.
 *
 * @return mixed
 *   The value to assign to the element.
 */
function cdek_select_element_value_callback(array $element, $input = FALSE) {
  $value = [];

  if ($input === FALSE) {
    if (isset($element['#default_value'])) {
      $point = CdekApi::getInstance()->getPickupPoint($element['#default_value']);

      if ($point !== NULL) {
        $value = [
          'country' => $point->CountryCode,
          'region' => $point->RegionCode,
          'city' => $point->CityCode,
          'point' => $point->Code,
        ];
      }
    }
  }
  elseif (is_array($input)) {
    $value = $input;
  }
  return $value;
}

/**
 * Processes a cdek_select form element.
 *
 * @param array $element
 *   The form element to process.
 *
 * @return array
 *   The processed element.
 */
function cdek_select_element_process(array $element) {
  $value = is_array($element['#value']) ? $element['#value'] : [];
  $value += array_fill_keys(['country', 'region', 'city', 'point'], '');
  $is_value_locked = FALSE;

  $request = cdek_select_element_resolve_request($element);
  $params = $request->getParams();

  $hide_city = isset($params['cityid']);
  $hide_region = isset($params['regionid']) || $hide_city;
  $hide_country = isset($params['countryid']) || $hide_region;

  // Generate a unique wrapper HTML ID.
  $ajax_wrapper_id = drupal_html_id('ajax-wrapper');
  $ajax_settings = [
    'callback' => 'cdek_select_element_ajax_update_callback',
    'options' => [
      'query' => [
        'element_parents' => implode('/', $element['#array_parents']),
      ],
    ],
    'wrapper' => $ajax_wrapper_id,
    'effect' => 'fade',
  ];

  // Element to select the country.
  $element['country'] = $element['country'] ?? [];
  $element['country'] = cdek_select_element_process_nested_element($element['country'], [
    '#type' => 'select',
    '#title' => t('Country'),
    '#ajax' => $ajax_settings,
    '#access' => !$hide_country,
    '#weight' => 0,
  ]);

  if ($element['country']['#access']) {
    // The country selection does not depend on other elements.
    $countries = CdekApi::getInstance()->getCountries($request) ?: [];
    $country = isset($countries[$value['country']]) ? $value['country'] : '';

    $element['country']['#options'] = $countries;
    $element['country']['#value'] = $country;
    cdek_select_element_process_nested_value($element['country']);

    // If the country is not yet selected, block further selection.
    // Otherwise, add the country to the parameters.
    if ($element['country']['#value'] === '') {
      $is_value_locked = TRUE;
    }
    else {
      $request->setCountryId($value['country']);
    }
  }

  // Element to select the region.
  $element['region'] = $element['region'] ?? [];
  $element['region'] = cdek_select_element_process_nested_element($element['region'], [
    '#type' => 'select',
    '#title' => t('Region'),
    '#ajax' => $ajax_settings,
    '#access' => !$hide_region,
    '#weight' => 10,
  ]);

  if ($element['region']['#access']) {
    // The region selection may depend on the selected country.
    $regions = !$is_value_locked ? (CdekApi::getInstance()->getRegions($request) ?: []) : [];
    $region = isset($regions[$value['region']]) ? $value['region'] : '';

    $element['region']['#options'] = $regions;
    $element['region']['#value'] = $region;
    cdek_select_element_process_nested_value($element['region']);

    // If the region is not yet selected, block further selection.
    // Otherwise, add the region to the parameters.
    if ($element['region']['#value'] === '') {
      $is_value_locked = TRUE;
    }
    else {
      $request->setRegionId($value['region']);
    }
  }

  // Element to select the city.
  $element['city'] = $element['city'] ?? [];
  $element['city'] = cdek_select_element_process_nested_element($element['city'], [
    '#type' => 'select',
    '#title' => t('City'),
    '#ajax' => $ajax_settings,
    '#access' => !$hide_city,
    '#weight' => 20,
  ]);

  if ($element['city']['#access']) {
    // The city selection may depend on the selected country and region.
    $cities = !$is_value_locked ? (CdekApi::getInstance()->getCities($request) ?: []) : [];
    $city = isset($cities[$value['city']]) ? $value['city'] : '';

    $element['city']['#options'] = $cities;
    $element['city']['#value'] = $city;
    cdek_select_element_process_nested_value($element['city']);

    // If the city is not yet selected, block further selection.
    // Otherwise, add the city to the parameters.
    if ($element['city']['#value'] === '') {
      $is_value_locked = TRUE;
    }
    else {
      $request->setCityId($value['city']);
    }
  }

  // Element to select the point.
  $element['point'] = $element['point'] ?? [];
  $element['point'] = cdek_select_element_process_nested_element($element['point'], [
    '#type' => 'select',
    '#title' => t('Point'),
    '#weight' => 30,
  ]);

  // The point selection may depend on the selected country, region and city.
  $points = !$is_value_locked ? (CdekApi::getInstance()->getPickupPoints($request) ?: []) : [];
  $points = array_column($points, 'Name', 'Code');
  $point = isset($points[$value['point']]) ? $value['point'] : '';

  $element['point']['#options'] = $points;
  $element['point']['#value'] = $point;
  cdek_select_element_process_nested_value($element['point']);

  // Prefix and suffix used for Ajax replacement.
  if ($element['country']['#access'] || $element['region']['#access'] || $element['city']['#access']) {
    $element['#prefix'] = '<div id="' . $ajax_wrapper_id . '">';
    $element['#suffix'] = '</div>';
  }
  return $element;
}

/**
 * After-build callback for a cdek_select form element.
 *
 * @param array $element
 *   The element structure.
 * @param array $form_state
 *   The current state of the form.
 *
 * @return array
 *   The element structure.
 */
function cdek_select_element_after_build(array $element, array &$form_state) {
  $element['#value'] = $element['point']['#value'];
  form_set_value($element, $element['#value'], $form_state);
  return $element;
}

/**
 * Ajax callback to update cdek_select form element.
 *
 * @param array $form
 *   The form structure.
 *
 * @return array
 *   The element to be rendered.
 */
function cdek_select_element_ajax_update_callback(array $form) {
  // Get element parents from the request.
  $element_parents = explode('/', $_GET['element_parents']);
  // Sanitize element parents before using them.
  $element_parents = array_filter($element_parents, 'element_child');
  // Retrieve the element to be rendered.
  return drupal_array_get_nested_value($form, $element_parents);
}

/**
 * Processes the structure of a nested element.
 *
 * @param array $element
 *   The nested element to process.
 * @param array $default
 *   Default structure of the nested element.
 *
 * @return array
 *   The processed nested element.
 */
function cdek_select_element_process_nested_element(array $element, array $default) {
  return drupal_array_merge_deep_array([$default, $element]);
}

/**
 * Processes the value of a nested element.
 *
 * @param array $element
 *   The nested element to process.
 */
function cdek_select_element_process_nested_value(array &$element) {
  if ($element['#type'] == 'select') {
    $element['#empty_value'] = '';
  }
  if ($element['#type'] == 'radios') {
    $element['#default_value'] = $element['#value'];
  }
}

/**
 * Resolves the request object.
 *
 * @param array $element
 *   The element structure.
 *
 * @return \CdekSDK\Requests\PvzListRequest
 *   An instance of the request object.
 */
function cdek_select_element_resolve_request(array $element) {
  $request = $element['#cdek_request'];
  if ($request instanceof PvzListRequest) {
    return clone $request;
  }
  else {
    return new PvzListRequest();
  }
}

/**
 * Value callback for a cdek_widget form element.
 *
 * @param array $element
 *   An associative array containing the properties of the element.
 * @param mixed $input
 *   The incoming input to populate the form element. If this is FALSE,
 *   the element's default value should be returned.
 *
 * @return mixed
 *   The value to assign to the element.
 */
function cdek_widget_element_value_callback(array $element, $input = FALSE) {
  $value = '';

  if ($input === FALSE) {
    if (isset($element['#default_value'])) {
      $point = CdekApi::getInstance()->getPickupPoint($element['#default_value']);

      if ($point !== NULL) {
        $value = $point->Code;
      }
    }
  }
  elseif (is_scalar($input)) {
    $value = $input;
  }
  return $value;
}

/**
 * Processes a cdek_widget form element.
 *
 * @param array $element
 *   The form element to process.
 *
 * @return array
 *   The processed element.
 */
function cdek_widget_element_process(array $element) {
  $element['#theme_wrappers'][] = 'form_element';
  $element['#title_display'] = 'none';

  $element['item'] = [
    '#type' => 'item',
    '#required' => $element['#required'],
    '#input' => FALSE,
  ];
  $element['item']['item'] = [
    '#type' => 'html_tag',
    '#tag' => 'span',
    '#attributes' => [
      'class' => ['cdek-widget-item'],
    ],
  ];

  $element['value'] = [
    '#type' => 'hidden',
    '#parents' => $element['#parents'],
    '#attributes' => [
      'class' => ['cdek-widget-value'],
    ],
  ];

  if (isset($element['#title'])) {
    $element['item']['#title'] = $element['#title'];
  }
  if (!empty($element['#value'])) {
    $point = CdekApi::getInstance()->getPickupPoint($element['#value']);

    if ($point !== NULL) {
      $element['item']['item']['#value'] = $point->Name;
      $element['value']['#default_value'] = $point->Code;
    }
  }
  return $element;
}

/**
 * Pre-render callback for a cdek_widget form element.
 *
 * @param array $element
 *   The element structure.
 *
 * @return array
 *   The element structure.
 */
function cdek_widget_element_pre_render(array $element) {
  if (empty($element['#cdek_widget_id'])) {
    $element['#cdek_widget_id'] = drupal_random_key();
  }
  $widget_id = $element['#cdek_widget_id'];
  $tariff_list = $element['#cdek_widget_tariff_list'];
  $options = cdek_widget_element_build_options($element);

  // Store tariffs in the cache for later use in the widget controller.
  // @see cdek_api_widget_action()
  if (!empty($tariff_list)) {
    $cid = 'tariff_list:' . $widget_id;
    $expire = REQUEST_TIME + 21600;
    cache_set($cid, $tariff_list, 'cache_cdek_api', $expire);
  }

  // When the widget is considered popup.
  if ($options['popup']) {
    $element['popup'] = [
      '#type' => 'link',
      '#title' => $element['#cdek_widget_popup_title'] ?? t('Show widget'),
      '#href' => current_path(),
      '#attributes' => [
        'class' => ['cdek-widget-popup'],
      ],
    ];
  }

  // When the widget is considered static.
  if ($options['link']) {
    $element['link'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'id' => $options['link'],
        'class' => ['cdek-widget-link'],
      ],
    ];
  }

  // Settings for widget script.
  $settings['cdekApi']['widgets'][$widget_id] = [
    'id' => $widget_id,
    'options' => $options,
  ];

  // Library to process the widget.
  $element['#attached']['library'][] = ['cdek_api', 'widget'];
  $element['#attached']['js'][] = ['type' => 'setting', 'data' => $settings];
  return $element;
}

/**
 * Builds the widget options.
 *
 * @param array $element
 *   The element structure.
 *
 * @return array
 *   The widget options.
 */
function cdek_widget_element_build_options(array $element) {
  $options = $element['#cdek_widget_options'];
  $widget_id = $element['#cdek_widget_id'];

  // Merge the default options.
  $options += [
    'hideMessages' => TRUE,
    'detailAddress' => FALSE,
    'popup' => FALSE,
  ];

  $options['link'] = $options['popup'] ? FALSE : 'cdek-widget-link-' . $widget_id;
  $options['servicepath'] = url('cdek-api/widget/action/' . $widget_id);

  if (empty($element['#processed'])) {
    $options['choose'] = FALSE;
  }
  else {
    $options['hidedelt'] = TRUE;
    $options['choose'] = TRUE;
  }
  return $options;
}
