<?php

/**
 * @file
 * Page callbacks for the CDEK API module.
 */

use CdekSDK\Requests\CalculationAuthorizedRequest;
use CdekSDK\Requests\CalculationRequest;
use CdekSDK\Requests\PvzListRequest;

/**
 * Widget action ID; Returns the pickup points data.
 */
const CDEK_API_WIDGET_ACTION_PVZ = 'getPVZ';

/**
 * Widget action ID; Returns translations for the widget.
 */
const CDEK_API_WIDGET_ACTION_LANG = 'getLang';

/**
 * Widget action ID; Returns the city ID by its name.
 */
const CDEK_API_WIDGET_ACTION_CITY = 'getCity';

/**
 * Widget action ID; Calculates shipping cost.
 */
const CDEK_API_WIDGET_ACTION_CALC = 'calc';

/**
 * Exception that indicates an error in the action of the pickup points widget.
 */
class CdekApiWidgetActionException extends RuntimeException {}

/**
 * Returns JSON representing the response to the specified widget action.
 *
 * @param string|null $widget_id
 *   The widget ID.
 */
function cdek_api_widget_action($widget_id = NULL) {
  $params = drupal_get_query_parameters();
  $action = $params['isdek_action'] ?? NULL;

  try {
    switch ($action) {
      case CDEK_API_WIDGET_ACTION_PVZ:
        $data = cdek_api_widget_action_pvz($params);
        break;

      case CDEK_API_WIDGET_ACTION_LANG:
        $data = cdek_api_widget_action_lang();
        break;

      case CDEK_API_WIDGET_ACTION_CITY:
        $data = cdek_api_widget_action_city($params);
        break;

      case CDEK_API_WIDGET_ACTION_CALC:
        $data = cdek_api_widget_action_calc($params, $widget_id);
        break;
    }
  }
  catch (CdekApiWidgetActionException $e) {
    $data = ['error' => $e->getMessage()];
  }
  drupal_json_output($data ?? []);
}

/**
 * Performs a widget action.
 *
 * Returns the pickup points data.
 *
 * @param array $params
 *   The query parameters of the current request.
 *
 * @return array
 *   The result of the action.
 *
 * @throws CdekApiWidgetActionException
 *   If the action fails.
 */
function cdek_api_widget_action_pvz(array $params) {
  $points_request = new PvzListRequest();
  $points_request->setType(PvzListRequest::TYPE_ALL);

  $lang = $params['lang'] ?? '';
  $country = $params['country'] ?? '';

  if ($lang !== '') {
    $points_request->setLanguage($lang);
  }
  if ($country !== '' && $country !== 'all') {
    $countries = CdekApi::getInstance()->getCountries($points_request);
    $country_id = array_search($country, $countries);

    if ($country_id !== FALSE) {
      $points_request->setCountryId($country_id);
    }
  }

  $points = CdekApi::getInstance()->getPickupPoints($points_request);
  if ($points === NULL) {
    throw new CdekApiWidgetActionException('Failed to get the list of pickup points');
  }

  // Prepare a data structure.
  $data = array_fill_keys(['PVZ', 'CITY', 'REGIONS', 'CITYFULL'], []);
  foreach ($points as $code => $point) {
    $item = [
      'Name' => $point->Name,
      'WorkTime' => $point->WorkTime,
      'Address' => $point->Address,
      'Phone' => $point->Phone,
      'Note' => $point->Note,
      'cX' => $point->coordX,
      'cY' => $point->coordY,
      'Dressing' => $point->IsDressingRoom,
      'Cash' => $point->HaveCashless,
      'Postamat' => strtolower($point->Type) == 'postomat',
      'Station' => $point->NearestStation,
      'Site' => $point->Site,
      'Metro' => $point->MetroStation,
      'AddressComment' => $point->AddressComment,
      'CityCode' => $point->CityCode,
      'Picture' => [],
    ];

    if (!empty($point->WeightLimit)) {
      $item['WeightLim'] = [
        'MIN' => $point->WeightLimit->getWeightMin(),
        'MAX' => $point->WeightLimit->getWeightMax(),
      ];
    }
    foreach ($point->OfficeImages as $office_image) {
      if (valid_url($office_image->getUrl(), TRUE)) {
        $item['Picture'][] = $office_image->getUrl();
      }
    }
    // Add item to the result data array.
    $data['PVZ'][$point->CityCode][$code] = $item;

    // Add additional information.
    if (!isset($data['CITY'][$point->CityCode])) {
      $city = trim(strtok($point->City, ',('));
      $country = $point->CountryName;
      $region = $point->RegionName;

      $data['CITY'][$point->CityCode] = $city;
      $data['CITYREG'][$point->CityCode] = $point->RegionCode;
      $data['REGIONSMAP'][$point->RegionCode][] = $point->CityCode;
      $data['CITYFULL'][$point->CityCode] = "$country $region $city";
      $data['REGIONS'][$point->CityCode] = "$region, $country";
    }
  }
  return ['pvz' => $data];
}

/**
 * Performs a widget action.
 *
 * Returns translations for the widget.
 *
 * @return array
 *   The result of the action.
 */
function cdek_api_widget_action_lang() {
  $options = ['context' => 'CDEK Widget'];

  $lang = [
    'COURIER' => t('Courier', [], $options),
    'PICKUP' => t('Pickup', [], $options),
    'DAY' => t('days', [], $options),
    'RUB' => t('RUB', [], $options),
    'CITYSEARCH' => t('Search for a city', [], $options),
    'PVZ' => t('Points of self-delivery', [], $options),
    'COUNTING' => t('Calculation', [], $options),
    'NO_AVAIL' => t('No shipping methods available', [], $options),
    'CHOOSE_TYPE_AVAIL' => t('Choose a shipping method', [], $options),
    'CHOOSE_OTHER_CITY' => t('Choose another location', [], $options),
    'TYPE_ADDRESS' => t('Specify address', [], $options),
    'TYPE_ADDRESS_HERE' => t('Specify shipping address', [], $options),
    'L_ADDRESS' => t('Address of self-delivery', [], $options),
    'L_TIME' => t('Working hours', [], $options),
    'L_WAY' => t('How to get to us', [], $options),
    'L_CHOOSE' => t('Choose', [], $options),
    'H_LIST' => t('List of self-delivery', [], $options),
    'H_PROFILE' => t('Shipping method', [], $options),
    'H_CASH' => t('Payment by card', [], $options),
    'H_DRESS' => t('Dressing room', [], $options),
    'H_POSTAMAT' => t('Postamats CDEK', [], $options),
    'H_SUPPORT' => t('Support', [], $options),
    'H_QUESTIONS' => t('If you have any questions,<br> you can ask them to our specialists', [], $options),
    'ADDRESS_WRONG' => t('Impossible to define address. Please, recheck the address.', [], $options),
    'ADDRESS_ANOTHER' => t('Read the new terms and conditions.', [], $options),
  ];
  return ['LANG' => $lang];
}

/**
 * Performs a widget action.
 *
 * Returns the city ID by its name.
 *
 * @param array $params
 *   The query parameters of the current request.
 *
 * @return array
 *   The result of the action.
 *
 * @throws CdekApiWidgetActionException
 *   If the action fails.
 */
function cdek_api_widget_action_city(array $params) {
  // TODO: Process 'address' parameter.
  if (!isset($params['city']) || $params['city'] === '') {
    throw new CdekApiWidgetActionException('No city to search given');
  }
  if (($cities = CdekApi::getInstance()->getCities()) === NULL) {
    throw new CdekApiWidgetActionException('Failed to get the list of cities');
  }
  if (($id = cdek_api_array_search($params['city'], $cities)) === FALSE) {
    throw new CdekApiWidgetActionException('No city found');
  }
  return ['id' => $id];
}

/**
 * Performs a widget action.
 *
 * Calculates shipping cost.
 *
 * @param array $params
 *   The query parameters of the current request.
 * @param string|null $widget_id
 *   The widget ID.
 *
 * @return array
 *   The result of the action.
 *
 * @throws CdekApiWidgetActionException
 *   If the action fails.
 */
function cdek_api_widget_action_calc(array $params, $widget_id = NULL) {
  $shipment = $params['shipment'] ?? NULL;

  if (!isset($shipment['type']) || $shipment['type'] === '') {
    throw new CdekApiWidgetActionException('Type not specified');
  }
  if (!isset($shipment['cityFromId']) || $shipment['cityFromId'] === '') {
    throw new CdekApiWidgetActionException('Sender city not specified');
  }
  if (!isset($shipment['cityToId']) || $shipment['cityToId'] === '') {
    throw new CdekApiWidgetActionException('Receiver city not specified');
  }
  if (empty($shipment['goods']) || !is_array($shipment['goods'])) {
    throw new CdekApiWidgetActionException('Goods not specified');
  }

  $calc_request = CdekApi::getInstance()->hasCredentials() ? new CalculationAuthorizedRequest() : new CalculationRequest();
  $calc_request->setSenderCityId($shipment['cityFromId']);
  $calc_request->setReceiverCityId($shipment['cityToId']);

  if (!empty($widget_id)) {
    $cid = 'tariff_list:' . $widget_id;
    $cache = cache_get($cid, 'cache_cdek_api');

    if (isset($cache->data[$shipment['type']])) {
      $tariff_list = array_values($cache->data[$shipment['type']]);
    }
  }

  if (!empty($tariff_list)) {
    foreach ($tariff_list as $priority => $tariff_id) {
      $calc_request->addTariffToList($tariff_id, $priority + 1);
    }
  }
  else {
    $calc_request->setTariffId(1);
  }

  foreach ($shipment['goods'] as $good) {
    $calc_request->addPackage($good);
  }

  try {
    $response = CdekApi::getInstance()->getCdekClient()
      ->sendCalculationRequest($calc_request);
  }
  catch (Exception $e) {
    throw new CdekApiWidgetActionException('Failed to calculate');
  }
  if ($response->hasErrors()) {
    throw new CdekApiWidgetActionException('Failed to calculate');
  }

  $delivery_date_min = $response->getDeliveryDateMin();
  if ($delivery_date_min instanceof DateTimeImmutable) {
    $delivery_date_min = $delivery_date_min->format('Y-m-d');
  }
  $delivery_date_max = $response->getDeliveryDateMax();
  if ($delivery_date_max instanceof DateTimeImmutable) {
    $delivery_date_max = $delivery_date_max->format('Y-m-d');
  }

  $result = [
    'price' => $response->getPrice(),
    'deliveryPeriodMin' => $response->getDeliveryPeriodMin(),
    'deliveryPeriodMax' => $response->getDeliveryPeriodMax(),
    'deliveryDateMin' => $delivery_date_min,
    'deliveryDateMax' => $delivery_date_max,
    'tariffId' => $response->getTariffId(),
  ];

  return [
    'type' => $shipment['type'],
    'timestamp' => $shipment['timestamp'] ?? '',
    'result' => $result,
  ];
}

/**
 * Case in-sensitive array_search() with partial matches.
 *
 * @param string $needle
 *   The string to search for.
 * @param array $haystack
 *   The array to search in.
 *
 * @return mixed
 *   The key for needle if it is found in the array, FALSE otherwise.
 */
function cdek_api_array_search($needle, array $haystack) {
  foreach ($haystack as $key => $value) {
    if (mb_stripos($value, $needle) === 0) {
      return $key;
    }
  }
  return FALSE;
}
