<?php

/**
 * @file
 * Contains the CdekApi class.
 */

use CdekSDK\Requests\PvzListRequest;
use CdekSDK\Common\Pvz;
use CdekSDK\CdekClient;
use GuzzleHttp\Client;

/**
 * Provides the service.
 */
class CdekApi {

  /**
   * The settings array.
   *
   * @var array
   */
  protected $config;

  /**
   * The CDEK API client.
   *
   * @var \CdekSDK\CdekClient
   */
  protected $cdekClient;

  /**
   * The instance of this class.
   *
   * @var CdekApi
   */
  protected static $instance;

  /**
   * Array of pickup points keyed by cache key.
   *
   * @var array
   */
  protected static $points = [];

  /**
   * Array of countries keyed by cache key.
   *
   * @var array
   */
  protected static $countries = [];

  /**
   * Array of regions keyed by cache key.
   *
   * @var array
   */
  protected static $regions = [];

  /**
   * Array of cities keyed by cache key.
   *
   * @var array
   */
  protected static $cities = [];

  /**
   * CdekApi constructor.
   */
  protected function __construct() {
    $this->config = static::getConfig();
  }

  /**
   * Gets the instance of this class.
   *
   * @return CdekApi
   *   The instance of this class.
   */
  public static function getInstance() {
    if (!isset(static::$instance)) {
      static::$instance = new static();
    }
    return static::$instance;
  }

  /**
   * Gets the settings array.
   *
   * @return array
   *   The settings array.
   */
  public static function getConfig() {
    $config = variable_get('cdek_api_settings', []);

    // Merge to defaults.
    $config += [
      'account' => '',
      'password' => '',
      'test_mode' => FALSE,
      'request_timeout' => 30,
      'cache_lifetime' => NULL,
    ];
    return $config;
  }

  /**
   * Gets the CDEK API client.
   *
   * @return \CdekSDK\CdekClient
   *   The CDEK API client.
   */
  public function getCdekClient() {
    if (!isset($this->cdekClient)) {
      $http_client = new Client([
        'base_uri' => $this->getBaseUri(),
        'timeout' => (float) $this->config['request_timeout'],
      ]);

      $account = (string) $this->config['account'];
      $password = (string) $this->config['password'];
      $this->cdekClient = new CdekClient($account, $password, $http_client);
    }
    return $this->cdekClient;
  }

  /**
   * Gets the list of pickup points.
   *
   * @param \CdekSDK\Requests\PvzListRequest|null $request
   *   An instance of the request object.
   *
   * @return \CdekSDK\Common\Pvz[]|null
   *   Array of pickup points keyed by code. NULL on failure.
   */
  public function getPickupPoints(PvzListRequest $request = NULL) {
    $request = $request ?? new PvzListRequest();
    $params = $request->getParams();
    $key = $this->getCacheKey($params);

    if (!isset(static::$points[$key])) {
      $cid = 'pickup_points:' . $key;
      $points = $this->getCacheData($cid);

      if (!isset($points)) {
        try {
          $response = $this->getCdekClient()->sendPvzListRequest($request);
        }
        catch (Exception $e) {
          return NULL;
        }

        $points = $response->getItems();
        // Use the point code as the key.
        foreach ($points as $index => $point) {
          $points[$point->Code] = $point;
          unset($points[$index]);
        }
        // Sort points alphabetically.
        uasort($points, [$this, 'comparePointsByName']);
        $this->setCacheData($cid, $points);
      }

      // Trigger hook_cdek_api_pickup_points_alter().
      // Allow modules to override the list of pickup points.
      drupal_alter('cdek_api_pickup_points', $points, $params);
      static::$points[$key] = $points;
    }
    return static::$points[$key];
  }

  /**
   * Gets the pickup point.
   *
   * @param string $code
   *   The code of the point to load.
   *
   * @return \CdekSDK\Common\Pvz|null
   *   The point object or NULL if there is no point with the given code.
   *
   * @see CdekApi::getPickupPoints()
   */
  public function getPickupPoint($code) {
    $points = $this->getPickupPoints();
    return $points[$code] ?? NULL;
  }

  /**
   * Gets the list of countries.
   *
   * @param \CdekSDK\Requests\PvzListRequest|null $request
   *   An instance of the request object.
   *
   * @return array|null
   *   Array of country names keyed by code. NULL on failure.
   *
   * @see CdekApi::getPickupPoints()
   */
  public function getCountries(PvzListRequest $request = NULL) {
    $request = $request ?? new PvzListRequest();
    $key = $this->getCacheKey($request->getParams());

    if (!isset(static::$countries[$key])) {
      $points = $this->getPickupPoints($request);
      if ($points === NULL) {
        return NULL;
      }

      // Extract countries from pickup points.
      $countries = [];
      foreach ($points as $point) {
        if (!isset($countries[$point->CountryCode])) {
          $countries[$point->CountryCode] = $point->CountryName;
        }
      }
      // Sort countries alphabetically.
      uasort($countries, [$this, 'strcasecmp']);
      static::$countries[$key] = $countries;
    }
    return static::$countries[$key];
  }

  /**
   * Gets the list of regions.
   *
   * @param \CdekSDK\Requests\PvzListRequest|null $request
   *   An instance of the request object.
   *
   * @return array|null
   *   Array of region names keyed by code. NULL on failure.
   *
   * @see CdekApi::getPickupPoints()
   */
  public function getRegions(PvzListRequest $request = NULL) {
    $request = $request ?? new PvzListRequest();
    $key = $this->getCacheKey($request->getParams());

    if (!isset(static::$regions[$key])) {
      $points = $this->getPickupPoints($request);
      if ($points === NULL) {
        return NULL;
      }

      // Extract regions from pickup points.
      $regions = [];
      foreach ($points as $point) {
        if (!isset($regions[$point->RegionCode])) {
          $regions[$point->RegionCode] = $point->RegionName;
        }
      }
      // Sort regions alphabetically.
      uasort($regions, [$this, 'strcasecmp']);
      static::$regions[$key] = $regions;
    }
    return static::$regions[$key];
  }

  /**
   * Gets the list of cities.
   *
   * @param \CdekSDK\Requests\PvzListRequest|null $request
   *   An instance of the request object.
   *
   * @return array|null
   *   Array of city names keyed by code. NULL on failure.
   *
   * @see CdekApi::getPickupPoints()
   */
  public function getCities(PvzListRequest $request = NULL) {
    $request = $request ?? new PvzListRequest();
    $key = $this->getCacheKey($request->getParams());

    if (!isset(static::$cities[$key])) {
      $points = $this->getPickupPoints($request);
      if ($points === NULL) {
        return NULL;
      }

      // Extract cities from pickup points.
      $cities = [];
      foreach ($points as $point) {
        if (!isset($cities[$point->CityCode])) {
          $cities[$point->CityCode] = $point->City;
        }
      }
      // Sort cities alphabetically.
      uasort($cities, [$this, 'strcasecmp']);
      static::$cities[$key] = $cities;
    }
    return static::$cities[$key];
  }

  /**
   * Determines whether credentials are set.
   *
   * @return bool
   *   TRUE if credentials are set and FALSE otherwise.
   */
  public function hasCredentials() {
    $account = (string) $this->config['account'];
    $password = (string) $this->config['password'];
    return $account !== '' && $password !== '';
  }

  /**
   * Determines whether a persistent cache is used.
   *
   * @return bool
   *   TRUE if a persistent cache is used and FALSE otherwise.
   */
  public function usesPersistentCache() {
    return $this->config['cache_lifetime'] !== NULL;
  }

  /**
   * Gets the cache key for a value.
   *
   * @param mixed $value
   *   The value to get the cache key.
   *
   * @return string
   *   The cache key.
   */
  protected function getCacheKey($value) {
    return drupal_hash_base64(serialize($value));
  }

  /**
   * Returns data from the persistent cache.
   *
   * @param string $cid
   *   The cache ID of the data to retrieve.
   *
   * @return mixed|null
   *   The cache data. NULL if no matching data found.
   */
  protected function getCacheData($cid) {
    if ($this->usesPersistentCache()) {
      $item = cache_get($cid, 'cache_cdek_api');
      return $item->data ?? NULL;
    }
    return NULL;
  }

  /**
   * Stores data in the persistent cache.
   *
   * @param string $cid
   *   The cache ID of the data to store.
   * @param mixed $data
   *   The data to store in the cache.
   */
  protected function setCacheData($cid, $data) {
    if ($this->usesPersistentCache()) {
      $expire = $this->config['cache_lifetime'];
      if ($expire !== CACHE_PERMANENT) {
        $expire = $expire * 60 + REQUEST_TIME;
      }
      cache_set($cid, $data, 'cache_cdek_api', $expire);
    }
  }

  /**
   * Gets the base URI of the client.
   *
   * @return string
   *   The base URI of the client.
   */
  protected function getBaseUri() {
    if ($this->config['test_mode']) {
      return 'https://integration.edu.cdek.ru';
    }
    return CdekClient::STANDARD_BASE_URL;
  }

  /**
   * Compares two pickup points by their names.
   *
   * @param \CdekSDK\Common\Pvz $point1
   *   The first pickup point.
   * @param \CdekSDK\Common\Pvz $point2
   *   The second pickup point.
   *
   * @return int
   *   Returns < 0 if $point1 is less than $point2; > 0 if $point1 is greater
   *   than $point2, and 0 if they are equal.
   */
  protected function comparePointsByName(Pvz $point1, Pvz $point2) {
    return $this->strcasecmp($point1->Name, $point2->Name);
  }

  /**
   * Compares UTF-8-encoded strings in a binary safe case-insensitive manner.
   *
   * @param string $str1
   *   The first string.
   * @param string $str2
   *   The second string.
   *
   * @return int
   *   Returns < 0 if $str1 is less than $str2; > 0 if $str1 is greater than
   *   $str2, and 0 if they are equal.
   */
  protected function strcasecmp($str1, $str2) {
    return strcmp(drupal_strtoupper($str1), drupal_strtoupper($str2));
  }

}
